@@ -405,13 +405,13 @@ class ImageRGBA(Glyph):
     The y-coordinates to locate the image anchors.
     """)
 
-    rows = NumberSpec(None, help="""
+    rows = NumberSpec(None, accept_datetime=False, help="""
     The numbers of rows in the images
-    """).asserts(False, lambda: deprecated((0, 12, 4), "ImageRGBA.rows", "2D array representation"))
+    """).asserts(False, lambda obj, name, value: deprecated((0, 12, 4), "ImageRGBA.rows", "2D array representation"))
 
-    cols = NumberSpec(None, help="""
+    cols = NumberSpec(None, accept_datetime=False, help="""
     The numbers of columns in the images
-    """).asserts(False, lambda: deprecated((0, 12, 4), "ImageRGBA.cols", "2D array representation"))
+    """).asserts(False, lambda obj, name, value: deprecated((0, 12, 4), "ImageRGBA.cols", "2D array representation"))
 
     dw = DistanceSpec(help="""
     The widths of the plot regions that the images will occupy.
@@ -451,7 +451,8 @@ class ImageURL(Glyph):
     # functions derived from this class
     _args = ('url', 'x', 'y', 'w', 'h', 'angle', 'global_alpha', 'dilate')
 
-    url = NumberSpec(help="""
+    # TODO (bev) Why is this a NumberSpec??
+    url = NumberSpec(accept_datetime=False, help="""
     The URLs to retrieve images from.
 
     .. note::
@@ -467,28 +468,18 @@ class ImageURL(Glyph):
     The y-coordinates to locate the image anchors.
     """)
 
-    # TODO: (bev) rename to "dw" for consistency
-    w = DistanceSpec(help="""
-    The widths of the plot regions that the images will occupy.
+    w = DistanceSpec(default=None, help="""
+    The height of the plot region that the image will occupy in data space.
 
-    .. note::
-        This is not the number of pixels that an image is wide.
-        That number is fixed by the image itself.
-
-    .. note::
-        This may be renamed to "dw" in the future.
+    The default value is ``None``, in which case the image will be displayed
+    at its actual image size (regardless of the units specified here).
     """)
 
-    # TODO: (bev) rename to "dh" for consistency
-    h = DistanceSpec(help="""
-    The height of the plot region that the image will occupy.
+    h = DistanceSpec(default=None, help="""
+    The height of the plot region that the image will occupy in data space.
 
-    .. note::
-        This is not the number of pixels that an image is tall.
-        That number is fixed by the image itself.
-
-    .. note::
-        This may be renamed to "dh" in the future.
+    The default value is ``None``, in which case the image will be displayed
+    at its actual image size (regardless of the units specified here).
     """)
 
     angle = AngleSpec(default=0, help="""
